
#ifndef _IUP_TREE_UTIL_H_
#define _IUP_TREE_UTIL_H_


#include <iup.h>
#include <iuptree.h>
#include <list>
#include <vector>

class IupLeafWrapper;
class IupTreeWrapper;

class IupNodeWrapper
{
public:
  IupNodeWrapper(const char* name, bool is_leaf, void* data = 0);
  virtual ~IupNodeWrapper();

  IupNodeWrapper* getParent() { return _parent; }
  bool isLeaf() { return _is_leaf; }

  // Returns the Id of the leaf
  // Not cached, dynamically calculated in each call.
  int id();

  // You can NOT change USERDATA or USERID attributes, 
  // or the wrapper will stop working.
  // Use this function instead.
  void setData(void* data) { _data = data; }
  void* getData() { return _data; }

  char* getName() { return _name; }
  void setName(const char* name);
  void updateName(const char* name);

  void setImage(const char* name);
  void setMarked(bool mark);
  bool isMarked();
  void setColor(int r,int g,int b);
  int getDepth();

  // It will automatically add the id of the node to the attribute name
  // Can be called only after the node has been inserted in an actual tree
  void SetAttribute(const char* name, const char* value);
  void SetAttributeHandle(const char* name, Ihandle* ih);
  char* GetAttribute(const char* name);

  // callbacks
  virtual int selection(int status);
  virtual int rename(char* name);
  virtual int showRename();
  virtual int rightClick();

protected:

  IupNodeWrapper* _parent;

  void setTree(Ihandle* tree) { _tree = tree; }

  void addNode(Ihandle* tree, IupNodeWrapper* parent, int id);
  void removeNode();

  Ihandle* _tree;
  char* _name;
  void* _data;
  bool _is_leaf;
};

class IupLeafWrapper: public IupNodeWrapper
{
  friend class IupTreeWrapper;

public:
  IupLeafWrapper(const char* name, void* data = 0)
    :IupNodeWrapper(name, true, data){};
  virtual ~IupLeafWrapper() {};

  // callbacks
  virtual int execute(char* name);
};


class IupTreeWrapper: public IupNodeWrapper
{
public:
  IupTreeWrapper(Ihandle* tree, const char* rootName, void* data = 0);
  IupTreeWrapper(const char* name, void* data = 0)
    :IupNodeWrapper(name, false, data){};
  virtual ~IupTreeWrapper() {};

  // Unnecessary since IUP 3, does nothing.
  void redraw() {};

  // Addition of leaves and branches
  // Limitation: can NOT add between leafs or branches, only at front or back
  // Limitation: can NOT use ADD*, INSERT*, COPYNODE, MOVENODE, or SHOWDRAGDROP attributes
  void addLeafFront(IupLeafWrapper* leaf);
  void addLeafBack(IupLeafWrapper* leaf);
  void addSubTreeFront(IupTreeWrapper* subTree);
  void addSubTreeBack(IupTreeWrapper* subTree);

  //
  IupLeafWrapper* findLeaf(void* data, bool searchSubTrees = true);

  // removal of leaves and branches
  bool removeLeaf(IupLeafWrapper* leaf); // the leaf pointer is NOT delete
  bool removeLeaves(bool includeSubTrees = true, bool deleteLeaves = false);  // the leaf pointer are deleted
  bool removeSubTrees(bool deleteSubTrees = false, bool deleteLeaves = false);  // the leaf pointer are deleted
  bool removeSubTree(IupTreeWrapper* tree, bool deleteSubTrees = false, bool deleteLeaves = false);  // the subtree pointer is NOT delete

  // callbacks
  virtual int branchOpen();
  virtual int branchClose();

protected:

  // Limitation: all branches comes first then all leafs, on the current branch
  typedef std::list<IupTreeWrapper*> TreeList;
  typedef std::list<IupLeafWrapper*> LeafList;

  TreeList _subTrees;
  LeafList _leaves;

  int subTreeFrontPosition();
  int subTreeBackPosition();
  int leafFrontPosition();
  int leafBackPosition();
};


#endif


